/*
 * a pointer to the function named msg_handler is passed to handler_init();
 * this is incorrectly interpreted as an invocation of msg_handler()
 */

/**
 * pointer to function taking a void* param, returning ulong
 */
unsigned long (* callback_fcn)(void* param);
/**
 * set callback_fcn to point to a particular function
 * \param handler function pointer
 */
static void handler_init (unsigned long (* handler)(void* param));
/**
 * invokes the callback_fcn
 * \param param general purpose pointer
 */
static void handle_msg (void *param);
/**
 * the function to be invoked through the callback_fcn variable
 * \param param general purpose pointer
 */
static unsigned long msg_handler (void *param);

/**
 * a function to kick everything off
 *
 * \param argc number of elements in the array
 * \param argv pointer to an array of character strings to be processed
 * \return 0 for success; anything else for anything else
 */
int main_moo (int argc, char *argv[])
{
    handler_init(msg_handler);
    handle_msg("some text");
    return 0;
}

/**
 * a callback function that does nothing
 *
 * \param param pointer to a chunk of memory
 * \return constant success indicator
 */
static unsigned long msg_handler (void *param)
{
    (void) param;
    return 1U;
}

/**
 * a function that accepts a pointer to a function (a MESSAGE_HANDLER)
 * and saves it for future use.
 *
 * \param callback pointer to a function
 */
static void handler_init (unsigned long (* callback)(void* param))
{
    /* set up callbacks */
    callback_fcn = callback;
}

/**
 * a function to invoke a saved MESSAGE_HANDLER
 *
 * \param param pointer to a chunk of generic memory
 */
static void handle_msg (void *param)
{
    (*callback_fcn)(param);
}
